module.exports={A:{A:{"1":"A B","2":"K D E F aC"},B:{"1":"C L M G N O P","516":"1 2 3 4 5 6 7 8 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I"},C:{"132":"hB iB jB kB lB mB nB EC oB FC pB qB rB","164":"0 bC DC J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB eC fC","516":"sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a","1028":"1 2 3 4 5 6 7 8 9 b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC"},D:{"420":"0 J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB","516":"1 2 3 4 5 6 7 8 9 fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC"},E:{"1":"A B C L M G KC 7B 8B lC mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC","132":"F kC","164":"D E jC","420":"J FB K gC JC hC iC"},F:{"1":"C 7B YC vC 8B","2":"F B rC sC tC uC","420":"0 G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB","516":"SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u"},G:{"1":"3C 4C 5C 6C 7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC","132":"1C 2C","164":"E zC 0C","420":"JC wC ZC xC yC"},H:{"1":"ID"},I:{"420":"DC J JD KD LD MD ZC ND OD","516":"I"},J:{"420":"D A"},K:{"1":"C 7B YC 8B","2":"A B","516":"H"},L:{"516":"I"},M:{"1028":"6B"},N:{"1":"A B"},O:{"516":"9B"},P:{"420":"J","516":"0 v w x y z PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD"},Q:{"516":"aD"},R:{"516":"bD"},S:{"164":"cD dD"}},B:4,C:"CSS3 Multiple column layout",D:true};
