"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _readOnlyError;
function _readOnlyError(name) {
  throw new TypeError('"' + name + '" is read-only');
}

//# sourceMappingURL=readOnlyError.js.map
