module.exports={A:{A:{"2":"K D E F A B aC"},B:{"1":"9 AB BB CB DB EB I","2":"C L M G N O P Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r","194":"s t u","516":"1 2 3 4 5 6 7 8"},C:{"1":"6 7 8 9 AB BB CB DB EB I 6B HC IC cC dC","2":"0 1 2 3 bC DC J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u eC fC","322":"4 5"},D:{"1":"9 AB BB CB DB EB I 6B HC IC","2":"0 J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r","194":"s t u","516":"1 2 3 4 5 6 7 8"},E:{"1":"TC UC VC WC XC CC qC","2":"J FB K D E F A B C L M G gC JC hC iC jC kC KC 7B 8B lC mC nC LC MC 9B oC AC NC OC PC QC","516":"RC pC BC SC"},F:{"1":"p q r s t u","2":"0 F B C G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d rC sC tC uC 7B YC vC 8B","194":"e f g","516":"h i j k l m n o"},G:{"1":"TC UC VC WC XC CC","2":"E JC wC ZC xC yC zC 0C 1C 2C 3C 4C 5C 6C 7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC","516":"RC HD BC SC"},H:{"2":"ID"},I:{"1":"I","2":"DC J JD KD LD MD ZC ND OD"},J:{"2":"D A"},K:{"1":"H","2":"A B C 7B YC 8B"},L:{"1":"I"},M:{"1":"6B"},N:{"2":"A B"},O:{"2":"9B"},P:{"2":"J v w x PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD","516":"0 y z"},Q:{"2":"aD"},R:{"2":"bD"},S:{"2":"cD dD"}},B:5,C:"CSS Nesting",D:true};
