module.exports={A:{A:{"2":"K D E aC","2561":"A B","2692":"F"},B:{"1":"1 2 3 4 5 6 7 8 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I","2561":"C L M G N O P"},C:{"1":"1 2 3 4 5 6 7 8 9 eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC","16":"bC","1537":"0 J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB fC","1796":"DC eC"},D:{"1":"1 2 3 4 5 6 7 8 9 tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC","16":"J FB K D E F A B C L M","1025":"QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB","1537":"0 G N O P GB v w x y z HB IB JB KB LB MB NB OB PB"},E:{"1":"M G lC mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC","16":"J FB K gC JC","1025":"D E F A B C iC jC kC KC 7B","1537":"hC","4097":"L 8B"},F:{"1":"hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u 8B","16":"F B C rC sC tC uC 7B YC","260":"vC","1025":"0 x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB","1537":"G N O P GB v w"},G:{"1":"AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC","16":"JC wC ZC","1025":"E 0C 1C 2C 3C 4C 5C 6C 7C","1537":"xC yC zC","4097":"8C 9C"},H:{"2":"ID"},I:{"16":"JD KD","1025":"I OD","1537":"DC J LD MD ZC ND"},J:{"1025":"A","1537":"D"},K:{"1":"A B C H 7B YC 8B"},L:{"1":"I"},M:{"1":"6B"},N:{"2561":"A B"},O:{"1":"9B"},P:{"1025":"0 J v w x y z PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD"},Q:{"1":"aD"},R:{"1":"bD"},S:{"1":"dD","1537":"cD"}},B:1,C:"input event",D:true};
