module.exports={A:{A:{"132":"K D E F A B aC"},B:{"132":"1 2 3 4 5 6 7 8 9 C L M G N O P Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I"},C:{"132":"0 1 2 3 4 5 6 7 8 9 bC DC J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC eC fC"},D:{"132":"1 2 3 4 5 6 7 8 9 J FB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC","388":"0 K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB"},E:{"132":"J FB K D E F A B C L M G gC JC hC iC jC kC KC 7B 8B lC mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC"},F:{"132":"0 F B C G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u rC sC tC uC 7B YC vC 8B"},G:{"132":"E JC wC ZC xC yC zC 0C 1C 2C 3C 4C 5C 6C 7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC"},H:{"132":"ID"},I:{"132":"DC J I JD KD LD MD ZC ND OD"},J:{"132":"D A"},K:{"132":"A B C H 7B YC 8B"},L:{"132":"I"},M:{"132":"6B"},N:{"132":"A B"},O:{"132":"9B"},P:{"132":"0 J v w x y z PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD"},Q:{"132":"aD"},R:{"132":"bD"},S:{"132":"cD dD"}},B:6,C:"DNSSEC and DANE",D:true};
