module.exports={A:{A:{"1":"A B","2":"K D E F aC"},B:{"1":"1 2 3 4 5 6 7 8 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I","2":"C L M G N O P"},C:{"1":"1 2 3 4 5 6 7 8 9 SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC","2":"bC DC eC fC","132":"0 J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB"},D:{"1":"1 2 3 4 5 6 7 8 9 HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC","2":"J","16":"0 FB K D E w x y z","132":"F A B C L M G N O P GB v"},E:{"1":"C L M G 7B 8B lC mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC","2":"J FB gC JC hC","132":"K D E F A B iC jC kC KC"},F:{"1":"0 G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u","2":"F B C rC sC tC uC 7B YC vC 8B"},G:{"2":"yC zC","132":"E 0C 1C 2C 3C 4C 5C 6C 7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC","514":"JC wC ZC xC"},H:{"2":"ID"},I:{"2":"JD KD LD","260":"DC J MD ZC","514":"I ND OD"},J:{"132":"A","260":"D"},K:{"2":"A B C 7B YC 8B","514":"H"},L:{"260":"I"},M:{"2":"6B"},N:{"514":"A","1028":"B"},O:{"2":"9B"},P:{"260":"0 J v w x y z PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD"},Q:{"260":"aD"},R:{"260":"bD"},S:{"1":"cD dD"}},B:1,C:"accept attribute for file input",D:true};
