module.exports={A:{A:{"2":"K D E F A aC","132":"B"},B:{"1":"C L M G N O P","4":"1 2 3 4 5 6 7 8 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I"},C:{"2":"bC DC eC","4":"0 1 2 3 4 5 6 7 8 9 K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC","8":"J FB fC"},D:{"2":"J FB K","4":"0 1 2 3 4 5 6 7 8 9 D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC"},E:{"2":"J FB K D E F A B C L M G gC JC hC iC jC kC KC 7B 8B lC mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC"},F:{"2":"F B C rC sC tC uC 7B YC vC 8B","4":"0 G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u"},G:{"2":"JC wC","4":"E ZC xC yC zC 0C 1C 2C 3C 4C 5C 6C 7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC"},H:{"2":"ID"},I:{"2":"JD KD LD","4":"DC J I MD ZC ND OD"},J:{"2":"D","4":"A"},K:{"1":"C 8B","2":"A B 7B YC","4":"H"},L:{"4":"I"},M:{"4":"6B"},N:{"1":"B","2":"A"},O:{"4":"9B"},P:{"4":"0 J v w x y z PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD"},Q:{"4":"aD"},R:{"4":"bD"},S:{"4":"cD dD"}},B:4,C:"DeviceOrientation & DeviceMotion events",D:true};
