module.exports={A:{A:{"388":"A B","900":"K D E F aC"},B:{"388":"C L M G N O P","641":"1 2 3 4 5 6 7 8 9 r s t u AB BB CB DB EB I","900":"Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q"},C:{"772":"1 2 3 4 5 6 7 8 9 sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC","900":"0 bC DC J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB eC fC"},D:{"641":"1 2 3 4 5 6 7 8 9 r s t u AB BB CB DB EB I 6B HC IC","900":"0 J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q"},E:{"772":"A","900":"J FB K D E F B C L M G gC JC hC iC jC kC KC 7B 8B lC mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC"},F:{"16":"F rC","129":"B C sC tC uC 7B YC vC 8B","641":"d e f g h i j k l m n o p q r s t u","900":"0 G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c"},G:{"900":"E JC wC ZC xC yC zC 0C 1C 2C 3C 4C 5C 6C 7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC"},H:{"129":"ID"},I:{"641":"I","900":"DC J JD KD LD MD ZC ND OD"},J:{"900":"D A"},K:{"129":"A B C 7B YC 8B","641":"H"},L:{"900":"I"},M:{"772":"6B"},N:{"388":"A B"},O:{"900":"9B"},P:{"641":"0 w x y z","900":"J v PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD"},Q:{"900":"aD"},R:{"900":"bD"},S:{"772":"dD","900":"cD"}},B:2,C:"CSS page-break properties",D:true};
