module.exports={A:{A:{"2436":"K D E F A B aC"},B:{"260":"O P","2436":"C L M G N","8196":"1 2 3 4 5 6 7 8 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I"},C:{"2":"bC DC J FB K D E F A B C L M G N O P GB v w eC fC","772":"0 x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB","4100":"1 2 3 4 5 6 7 8 9 WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC"},D:{"2":"J FB K D E F A B C","2564":"0 L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB","8196":"1 2 3 4 5 6 7 8 9 nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC","10244":"YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB"},E:{"1":"C L M G 8B lC mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC","16":"gC JC","2308":"A B KC 7B","2820":"J FB K D E F hC iC jC kC"},F:{"2":"F B rC sC tC uC 7B YC vC","16":"C","516":"8B","2564":"0 G N O P GB v w x y z HB IB JB KB","8196":"aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u","10244":"LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB"},G:{"1":"7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC","2":"JC wC ZC","2820":"E xC yC zC 0C 1C 2C 3C 4C 5C 6C"},H:{"2":"ID"},I:{"2":"DC J JD KD LD MD ZC","260":"I","2308":"ND OD"},J:{"2":"D","2308":"A"},K:{"2":"A B C 7B YC","16":"8B","8196":"H"},L:{"8196":"I"},M:{"1028":"6B"},N:{"2":"A B"},O:{"8196":"9B"},P:{"2052":"PD QD","2308":"J","8196":"0 v w x y z RD SD TD KC UD VD WD XD YD AC BC CC ZD"},Q:{"8196":"aD"},R:{"8196":"bD"},S:{"4100":"cD dD"}},B:5,C:"Synchronous Clipboard API",D:true};
