module.exports={A:{A:{"2":"K aC","2340":"D E F A B"},B:{"2":"C L M G N O P","1025":"1 2 3 4 5 6 7 8 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I"},C:{"1":"1 2 3 4 5 6 7 8 9 c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC","2":"bC DC eC","513":"sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b","545":"0 J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB fC"},D:{"2":"0 J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB","1025":"1 2 3 4 5 6 7 8 9 WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC"},E:{"1":"A B C L M G KC 7B 8B lC mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC","2":"J FB gC JC hC","164":"K","4644":"D E F iC jC kC"},F:{"2":"0 F B G N O P GB v w x y z HB IB rC sC tC uC 7B YC","545":"C vC 8B","1025":"JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u"},G:{"1":"3C 4C 5C 6C 7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC","2":"JC wC ZC","4260":"xC yC","4644":"E zC 0C 1C 2C"},H:{"2":"ID"},I:{"2":"DC J JD KD LD MD ZC ND OD","1025":"I"},J:{"2":"D","4260":"A"},K:{"2":"A B 7B YC","545":"C 8B","1025":"H"},L:{"1025":"I"},M:{"1":"6B"},N:{"2340":"A B"},O:{"1025":"9B"},P:{"1025":"0 J v w x y z PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD"},Q:{"1025":"aD"},R:{"1025":"bD"},S:{"1":"dD","4097":"cD"}},B:4,C:"Crisp edges/pixelated images",D:true};
