import { parse as babelParse } from "@babel/parser";
import { Overrides } from "./_babel_options";
type BabelParser = {
    parse: typeof babelParse;
};
export declare const parser: BabelParser;
export declare function parse(source: string, options?: Overrides): import("@babel/parser").ParseResult<import("@babel/types").File>;
export {};
