module.exports = {
  printWidth: 80,
  trailingComma: "all",
  singleQuote: false,

  overrides: [
    {
      files: "*.md",
      options: {
        printWidth: 60,
        // Don't reformat code examples in README
        embeddedLanguageFormatting: "off",
      },
    },
  ],
};
