module.exports={A:{A:{"2":"K D E F A B aC"},B:{"1":"1 2 3 4 5 6 7 8 9 L M G N O P Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I","132":"C"},C:{"2":"0 bC DC J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB eC fC","1090":"iB jB kB lB","2052":"mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b","4100":"1 2 3 4 5 6 7 8 9 c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC"},D:{"1":"0 1 2 3 4 5 6 7 8 9 HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC","2":"J FB K D E F A B C L M G N O P GB","2052":"v w x y z"},E:{"2":"J FB K D E F A B C L M gC JC hC iC jC kC KC 7B 8B lC","4100":"G mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC"},F:{"1":"0 F B C G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u rC sC tC uC 7B YC vC 8B"},G:{"2":"JC wC ZC","260":"E xC yC zC 0C 1C 2C 3C 4C 5C 6C 7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC"},H:{"2":"ID"},I:{"1":"I ND OD","2":"DC JD KD LD","514":"J MD ZC"},J:{"1":"A","2":"D"},K:{"1":"A B C H 7B YC 8B"},L:{"1":"I"},M:{"4100":"6B"},N:{"2":"A B"},O:{"1":"9B"},P:{"1":"0 J v w x y z PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD"},Q:{"1":"aD"},R:{"1":"bD"},S:{"2052":"cD dD"}},B:1,C:"Date and time input types",D:true};
