"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _temporalRef;
var _temporalUndefined = require("./temporalUndefined.js");
var _tdz = require("./tdz.js");
function _temporalRef(val, name) {
  return val === _temporalUndefined.default ? (0, _tdz.default)(name) : val;
}

//# sourceMappingURL=temporalRef.js.map
