"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _defaults;
function _defaults(obj, defaults) {
  for (var keys = Object.getOwnPropertyNames(defaults), i = 0; i < keys.length; i++) {
    var key = keys[i],
      value = Object.getOwnPropertyDescriptor(defaults, key);
    if (value && value.configurable && obj[key] === undefined) {
      Object.defineProperty(obj, key, value);
    }
  }
  return obj;
}

//# sourceMappingURL=defaults.js.map
