"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _classPrivateMethodGet;
var _assertClassBrand = require("assertClassBrand");
function _classPrivateMethodGet(receiver, privateSet, fn) {
  _assertClassBrand(privateSet, receiver);
  return fn;
}

//# sourceMappingURL=classPrivateMethodGet.js.map
