module.exports={A:{A:{"1":"K D E F A aC","132":"B"},B:{"132":"C L M G N O P","260":"1 2 3 4 5 6 7 8 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I"},C:{"1":"0 bC DC J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB eC fC","516":"1 2 3 4 5 6 7 8 9 LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC"},D:{"1":"0 O P GB v w x y z HB","2":"J FB K D E F A B C L M G N","132":"IB JB KB LB MB NB OB PB QB RB SB TB UB VB","260":"1 2 3 4 5 6 7 8 9 WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC"},E:{"1":"K hC iC","2":"J FB gC JC","2052":"D E F A B C L M G jC kC KC 7B 8B lC mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC"},F:{"1":"0 F B C G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u rC sC tC uC 7B YC vC 8B"},G:{"2":"JC wC ZC","1025":"E xC yC zC 0C 1C 2C 3C 4C 5C 6C 7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC"},H:{"1025":"ID"},I:{"1":"DC J I JD KD LD MD ZC ND OD"},J:{"1":"D A"},K:{"1":"A B C H 7B YC 8B"},L:{"1":"I"},M:{"1":"6B"},N:{"2052":"A B"},O:{"1025":"9B"},P:{"1":"0 J v w x y z PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD"},Q:{"260":"aD"},R:{"1":"bD"},S:{"516":"cD dD"}},B:1,C:"autocomplete attribute: on & off values",D:true};
