module.exports={A:{A:{"2":"K D E F A B aC"},B:{"1":"1 2 3 4 5 6 7 8 9 Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I","2":"C L M G N O P","1028":"W X","1540":"Q H R S T U V"},C:{"1":"1 2 3 4 5 6 7 8 9 tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC","2":"bC","164":"0 DC J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB eC fC","1540":"WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB"},D:{"1":"1 2 3 4 5 6 7 8 9 Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC","292":"0 J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB","1028":"W X","1540":"wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V"},E:{"1":"G nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC","292":"J FB K D E F A B C gC JC hC iC jC kC KC 7B","1540":"L M 8B lC","3076":"mC"},F:{"1":"3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u","2":"F B C rC sC tC uC 7B YC vC 8B","292":"0 G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB","1028":"1B 2B","1540":"lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B"},G:{"1":"FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC","292":"E JC wC ZC xC yC zC 0C 1C 2C 3C 4C 5C 6C 7C","1540":"8C 9C AD BD CD DD","3076":"ED"},H:{"2":"ID"},I:{"1":"I","292":"DC J JD KD LD MD ZC ND OD"},J:{"292":"D A"},K:{"1":"H","2":"A B C 7B YC 8B"},L:{"1":"I"},M:{"1":"6B"},N:{"2":"A B"},O:{"1":"9B"},P:{"1":"0 v w x y z YD AC BC CC ZD","292":"J PD QD RD SD TD","1540":"KC UD VD WD XD"},Q:{"1540":"aD"},R:{"1":"bD"},S:{"1":"dD","1540":"cD"}},B:5,C:"CSS Logical Properties",D:true};
