"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _instanceof;
function _instanceof(left, right) {
  if (right != null && typeof Symbol !== "undefined" && right[Symbol.hasInstance]) {
    return !!right[Symbol.hasInstance](left);
  } else {
    return left instanceof right;
  }
}

//# sourceMappingURL=instanceof.js.map
