module.exports={A:{A:{"2":"K D E F A B aC"},B:{"2":"C L M G N O P","1025":"1 2 3 4 5 6 7 8 9 d e f g h i j k l m n o p q r s t u AB BB CB DB EB I","1537":"Q H R S T U V W X Y Z a b c"},C:{"2":"bC","932":"0 DC J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB eC fC","2308":"1 2 3 4 5 6 7 8 9 tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC"},D:{"2":"J FB K D E F A B C L M G N O P GB v w","545":"0 x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB","1025":"1 2 3 4 5 6 7 8 9 d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC","1537":"bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c"},E:{"1":"AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC","2":"J FB K gC JC hC","516":"B C L M G 7B 8B lC mC nC LC MC 9B oC","548":"F A kC KC","676":"D E iC jC"},F:{"2":"F B C rC sC tC uC 7B YC vC 8B","513":"PB","545":"0 G N O P GB v w x y z HB IB JB KB LB MB NB","1025":"e f g h i j k l m n o p q r s t u","1537":"OB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d"},G:{"1":"AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC","2":"JC wC ZC xC yC","516":"DD ED FD LC MC 9B GD","548":"1C 2C 3C 4C 5C 6C 7C 8C 9C AD BD CD","676":"E zC 0C"},H:{"2":"ID"},I:{"2":"DC J JD KD LD MD ZC","545":"ND OD","1025":"I"},J:{"2":"D","545":"A"},K:{"2":"A B C 7B YC 8B","1025":"H"},L:{"1025":"I"},M:{"2308":"6B"},N:{"2":"A B"},O:{"1537":"9B"},P:{"545":"J","1025":"0 v w x y z BC CC ZD","1537":"PD QD RD SD TD KC UD VD WD XD YD AC"},Q:{"1537":"aD"},R:{"1537":"bD"},S:{"932":"cD","2308":"dD"}},B:5,C:"Intrinsic & Extrinsic Sizing",D:true};
