module.exports={A:{A:{"1":"A B","2":"K D E F aC"},B:{"1":"1 2 3 4 5 6 7 8 9 C L M G N O P Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I"},C:{"1":"0 1 2 3 4 5 6 7 8 9 J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC","2":"bC DC eC fC"},D:{"1":"0 1 2 3 4 5 6 7 8 9 J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC"},E:{"1":"FB K D E F A B C L M G hC iC jC kC KC 7B 8B lC mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC","132":"J gC JC"},F:{"1":"0 C G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u YC vC 8B","2":"F rC sC tC uC","132":"B 7B"},G:{"1":"E JC wC ZC xC yC zC 0C 1C 2C 3C 4C 5C 6C 7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC"},H:{"1":"ID"},I:{"1":"DC I JD KD LD ZC ND OD","4":"J MD"},J:{"1":"D A"},K:{"1":"B C H 7B YC 8B","2":"A"},L:{"1":"I"},M:{"1":"6B"},N:{"1":"A B"},O:{"1":"9B"},P:{"1":"0 J v w x y z PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD"},Q:{"1":"aD"},R:{"1":"bD"},S:{"1":"cD dD"}},B:1,C:"input placeholder attribute",D:true};
