module.exports={A:{D:{"1":"1 2 3 4 5 6 7 8 9 dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC","2":"0 J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB"},L:{"1":"I"},B:{"1":"1 2 3 4 5 6 7 8 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I","2":"C L M G N O P"},C:{"1":"1 2 3 4 5 6 7 8 9 fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC","2":"bC DC J FB K D E F eC fC","33":"0 A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB"},M:{"1":"6B"},A:{"2":"K D E F A B aC"},F:{"1":"QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u","2":"0 F B C G N O P GB v w x y z HB IB JB KB LB MB NB OB PB rC sC tC uC 7B YC vC 8B"},K:{"1":"H","2":"A B C 7B YC 8B"},E:{"1":"B C L M G 7B 8B lC mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC","2":"J FB gC JC hC qC","33":"K D E F A iC jC kC KC"},G:{"1":"5C 6C 7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC","2":"JC wC ZC xC","33":"E yC zC 0C 1C 2C 3C 4C"},P:{"1":"0 v w x y z PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD","2":"J"},I:{"1":"I","2":"DC J JD KD LD MD ZC ND OD"}},B:6,C:"plaintext from unicode-bidi",D:undefined};
