module.exports={A:{A:{"1":"B","4":"K D E F A aC"},B:{"1":"C L M G N","129":"1 2 3 4 5 6 7 8 9 O P Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I"},C:{"1":"0 K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB","4":"bC DC J FB eC fC","129":"1 2 3 4 5 6 7 8 9 gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC"},D:{"1":"aB bB cB dB eB fB gB hB iB jB","4":"J FB K","129":"0 1 2 3 4 5 6 7 8 9 D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC"},E:{"4":"J FB gC JC","129":"K D E F A B C L M G hC iC jC kC KC 7B 8B lC mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC"},F:{"1":"C NB OB PB QB RB SB TB UB VB WB 7B YC vC 8B","4":"F B rC sC tC uC","129":"0 G N O P GB v w x y z HB IB JB KB LB MB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u"},G:{"4":"JC wC ZC","129":"E xC yC zC 0C 1C 2C 3C 4C 5C 6C 7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC"},H:{"4":"ID"},I:{"4":"JD KD LD","129":"DC J I MD ZC ND OD"},J:{"129":"D A"},K:{"1":"C 7B YC 8B","4":"A B","129":"H"},L:{"129":"I"},M:{"129":"6B"},N:{"1":"B","4":"A"},O:{"129":"9B"},P:{"129":"0 J v w x y z PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD"},Q:{"129":"aD"},R:{"129":"bD"},S:{"1":"cD","129":"dD"}},B:1,C:"dataset & data-* attributes",D:true};
