module.exports={A:{A:{"2":"K D E F A aC","548":"B"},B:{"1":"1 2 3 4 5 6 7 8 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I","516":"C L M G N O P"},C:{"1":"1 2 3 4 5 6 7 8 9 rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC","2":"bC DC J FB K D E F eC fC","676":"0 A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB","1700":"cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB"},D:{"1":"1 2 3 4 5 6 7 8 9 yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC","2":"J FB K D E F A B C L M","676":"G N O P GB","804":"0 v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB"},E:{"1":"QC RC pC BC SC TC UC VC WC XC CC qC","2":"J FB gC JC","548":"MC 9B oC AC NC OC PC","676":"hC","804":"K D E F A B C L M G iC jC kC KC 7B 8B lC mC nC LC"},F:{"1":"rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u 8B","2":"F B C rC sC tC uC 7B YC vC","804":"0 G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB"},G:{"2":"E JC wC ZC xC yC zC 0C 1C 2C 3C 4C 5C 6C","2052":"7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC"},H:{"2":"ID"},I:{"2":"DC J I JD KD LD MD ZC ND OD"},J:{"2":"D","292":"A"},K:{"1":"H","2":"A B C 7B YC 8B"},L:{"1":"I"},M:{"1":"6B"},N:{"2":"A","548":"B"},O:{"1":"9B"},P:{"1":"0 v w x y z KC UD VD WD XD YD AC BC CC ZD","804":"J PD QD RD SD TD"},Q:{"1":"aD"},R:{"1":"bD"},S:{"1":"cD dD"}},B:1,C:"Fullscreen API",D:true};
