module.exports={A:{A:{"2":"F A B aC","8":"K D E"},B:{"2":"C L M G N O P","8":"Q H R S T U V W X Y Z a b c d e f","584":"g h i j k l m n o p q r","1025":"1 2 3 4 5 6 7 8 9 s t u AB BB CB DB EB I"},C:{"1":"0 1 2 3 4 5 6 7 8 9 J FB K D E F A B C L M G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u AB BB CB DB EB I 6B HC IC cC dC","129":"bC DC eC fC"},D:{"1":"z","8":"0 J FB K D E F A B C L M G N O P GB v w x y HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB EC oB FC pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R S T U V W X Y Z a b c d e f","584":"g h i j k l m n o p q r","1025":"1 2 3 4 5 6 7 8 9 s t u AB BB CB DB EB I 6B HC IC"},E:{"1":"A B C L M G KC 7B 8B lC mC nC LC MC 9B oC AC NC OC PC QC RC pC BC SC TC UC VC WC XC CC qC","260":"J FB K D E F gC JC hC iC jC kC"},F:{"2":"F","8":"0 G N O P GB v w x y z HB IB JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B Q H R GC","584":"S T U V W X Y Z a b c d","1025":"e f g h i j k l m n o p q r s t u","2052":"B C rC sC tC uC 7B YC vC 8B"},G:{"1":"E xC yC zC 0C 1C 2C 3C 4C 5C 6C 7C 8C 9C AD BD CD DD ED FD LC MC 9B GD AC NC OC PC QC RC HD BC SC TC UC VC WC XC CC","8":"JC wC ZC"},H:{"8":"ID"},I:{"8":"DC J JD KD LD MD ZC ND OD","1025":"I"},J:{"1":"A","8":"D"},K:{"8":"A B C 7B YC 8B","1025":"H"},L:{"1025":"I"},M:{"1":"6B"},N:{"2":"A B"},O:{"8":"9B"},P:{"1":"0 w x y z","8":"J v PD QD RD SD TD KC UD VD WD XD YD AC BC CC ZD"},Q:{"8":"aD"},R:{"8":"bD"},S:{"1":"cD dD"}},B:2,C:"MathML",D:true};
